@@ -1,34 +1,28 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 
 //! The certificate store holds and retrieves private keys and certificates from disk. It is responsible
 //! for checking certificates supplied by the remote end to see if they are valid and trusted or not.
-use opcua_types::service_types::ApplicationDescription;
-use opcua_types::status_code::StatusCode;
-use openssl::{
-    pkey,
-    x509,
-};
-use std::fs::{File, metadata};
+
+use std::fs::{metadata, File};
 use std::io::{Read, Write};
 use std::path::{Path, PathBuf};
 
+use openssl::{pkey, x509};
+
+use opcua_types::status_code::StatusCode;
+
 use crate::{
     pkey::PrivateKey,
     security_policy::SecurityPolicy,
-    x509::{X509, X509Data},
+    x509::{X509Data, X509},
 };
 
-/// The name that the server/client's application instance certificate is expected to be
-const OWN_CERTIFICATE_NAME: &str = "cert.der";
-/// The name that the server/client's application instance private key is expected to be
-const OWN_PRIVATE_KEY_NAME: &str = "private.pem";
-
-/// The directory holding the server/client's application instance cert
-const OWN_CERTIFICATE_DIR: &str = "own";
-/// The directory holding the server/client's application instance private key
-const OWN_PRIVATE_KEY_DIR: &str = "private";
+/// Default path to the applications own certificate
+const OWN_CERTIFICATE_PATH: &str = "own/cert.der";
+/// Default path to the applications own private key
+const OWN_PRIVATE_KEY_PATH: &str = "private/private.pem";
 /// The directory holding trusted certificates
 const TRUSTED_CERTS_DIR: &str = "trusted";
 /// The directory holding rejected certificates
@@ -37,15 +31,23 @@ const REJECTED_CERTS_DIR: &str = "rejected";
 /// The certificate store manages the storage of a server/client's own certificate & private key
 /// and the trust / rejection of certificates from the other end.
 pub struct CertificateStore {
-    /// Path to the certificate store on disk.
-    pub pki_path: PathBuf,
+    /// Path to the applications own certificate
+    own_certificate_path: PathBuf,
+    /// Path to the applications own private key
+    own_private_key_path: PathBuf,
+    /// Path to the certificate store on disk
+    pub(crate) pki_path: PathBuf,
     /// Timestamps of the cert are normally checked on the cert to ensure it cannot be used before
     /// or after its limits, but this check can be disabled.
-    pub check_time: bool,
+    check_time: bool,
+    /// This option lets you skip additional certificate validations (e.g. hostname, application
+    /// uri and the not before / after values). Certificates are always checked to see if they are
+    /// trusted and have a valid key length.
+    skip_verify_certs: bool,
     /// Ordinarily an unknown cert will be dropped into the rejected folder, but it can be dropped
     /// into the trusted folder if this flag is set. Certs in the trusted folder must still pass
     /// validity checks.
-    pub trust_unknown_certs: bool,
+    trust_unknown_certs: bool,
 }
 
 impl CertificateStore {
@@ -54,15 +56,30 @@ impl CertificateStore {
     /// location on disk.
     pub fn new(pki_path: &Path) -> CertificateStore {
         CertificateStore {
+            own_certificate_path: PathBuf::from(OWN_CERTIFICATE_PATH),
+            own_private_key_path: PathBuf::from(OWN_PRIVATE_KEY_PATH),
             pki_path: pki_path.to_path_buf(),
             check_time: true,
+            skip_verify_certs: false,
             trust_unknown_certs: false,
         }
     }
 
-    /// Sets up the certificate store, creates the path to it, and optionally creates a demo cert
-    pub fn new_with_keypair(pki_path: &Path, application_description: Option<ApplicationDescription>) -> (CertificateStore, Option<X509>, Option<PrivateKey>) {
-        let certificate_store = CertificateStore::new(pki_path);
+    pub fn new_with_x509_data<X>(
+        pki_path: &Path,
+        overwrite: bool,
+        cert_path: Option<&Path>,
+        pkey_path: Option<&Path>,
+        x509_data: Option<X>,
+    ) -> (CertificateStore, Option<X509>, Option<PrivateKey>)
+    where
+        X: Into<X509Data>,
+    {
+        let mut certificate_store = CertificateStore::new(pki_path);
+        if let (Some(cert_path), Some(pkey_path)) = (cert_path, pkey_path) {
+            certificate_store.own_certificate_path = cert_path.to_path_buf();
+            certificate_store.own_private_key_path = pkey_path.to_path_buf();
+        }
         let (cert, pkey) = if certificate_store.ensure_pki_path().is_err() {
             error!("Folder for storing certificates cannot be examined so server has no application instance certificate or private key.");
             (None, None)
@@ -70,9 +87,11 @@ impl CertificateStore {
             let result = certificate_store.read_own_cert_and_pkey();
             if let Ok((cert, pkey)) = result {
                 (Some(cert), Some(pkey))
-            } else if let Some(application_description) = application_description {
+            } else if let Some(x509_data) = x509_data {
                 info!("Creating sample application instance certificate and private key");
-                let result = certificate_store.create_and_store_application_instance_cert(&X509Data::from(application_description), false);
+                let x509_data = x509_data.into();
+                let result = certificate_store
+                    .create_and_store_application_instance_cert(&x509_data, overwrite);
                 if let Err(err) = result {
                     error!("Certificate creation failed, error = {}", err);
                     (None, None)
@@ -81,13 +100,28 @@ impl CertificateStore {
                     (Some(cert), Some(pkey))
                 }
             } else {
-                error!("Application instance certificate and private key could not be read - {}", result.unwrap_err());
+                error!(
+                    "Application instance certificate and private key could not be read - {}",
+                    result.unwrap_err()
+                );
                 (None, None)
             }
         };
         (certificate_store, cert, pkey)
     }
 
+    pub fn set_skip_verify_certs(&mut self, skip_verify_certs: bool) {
+        self.skip_verify_certs = skip_verify_certs;
+    }
+
+    pub fn set_trust_unknown_certs(&mut self, trust_unknown_certs: bool) {
+        self.trust_unknown_certs = trust_unknown_certs;
+    }
+
+    pub fn set_check_time(&mut self, check_time: bool) {
+        self.check_time = check_time;
+    }
+
     /// Reads a private key from a path on disk.
     pub fn read_pkey(path: &Path) -> Result<PrivateKey, String> {
         if let Ok(pkey_info) = metadata(path) {
@@ -105,18 +139,20 @@ impl CertificateStore {
 
     /// Reads the store's own certificate and private key
     pub fn read_own_cert_and_pkey(&self) -> Result<(X509, PrivateKey), String> {
-        let own_cert_path = self.own_cert_path();
-        if let Ok(cert) = CertificateStore::read_cert(&own_cert_path) {
-            let own_private_key_path = self.own_private_key_path();
-            CertificateStore::read_pkey(&own_private_key_path)
-                .map(|pkey| {
-                    (cert, pkey)
-                })
+        if let Ok(cert) = CertificateStore::read_cert(&self.own_certificate_path()) {
+            CertificateStore::read_pkey(&self.own_private_key_path())
+                .map(|pkey| (cert, pkey))
                 .map_err(|_| {
-                    format!("Cannot read pkey from path {:?}", own_private_key_path)
+                    format!(
+                        "Cannot read pkey from path {:?}",
+                        self.own_private_key_path()
+                    )
                 })
         } else {
-            Err(format!("Cannot read cert from path {:?}", own_cert_path))
+            Err(format!(
+                "Cannot read cert from path {:?}",
+                self.own_certificate_path()
+            ))
         }
     }
 
@@ -129,29 +165,41 @@ impl CertificateStore {
         }
     }
 
-    /// This function will use the supplied arguments to create an Application Instance Certificate
-    /// consisting of a X509v3 certificate and public/private key pair. The cert (including pubkey)
-    /// and private key will be written to disk under the pki path.
-    pub fn create_and_store_application_instance_cert(&self, args: &X509Data, overwrite: bool) -> Result<(X509, PrivateKey), String> {
-        // Create the cert and corresponding private key
+    /// Create a certificate and key pair to the specified locations
+    pub fn create_certificate_and_key(
+        args: &X509Data,
+        overwrite: bool,
+        cert_path: &Path,
+        pkey_path: &Path,
+    ) -> Result<(X509, PrivateKey), String> {
         let (cert, pkey) = X509::cert_and_pkey(args)?;
 
-        // Public cert goes under own/
-        let public_cert_path = CertificateStore::make_and_ensure_file_path(&self.own_cert_dir(), OWN_CERTIFICATE_NAME)?;
-        // Private key goes under private/
-        let private_key_path = CertificateStore::make_and_ensure_file_path(&self.private_key_dir(), OWN_PRIVATE_KEY_NAME)?;
-
         // Write the public cert
-        let _ = CertificateStore::store_cert(&cert, &public_cert_path, overwrite)?;
+        let _ = CertificateStore::store_cert(&cert, cert_path, overwrite)?;
 
         // Write the private key
         let pem = pkey.private_key_to_pem().unwrap();
-        info!("Writing private key to {}", private_key_path.display());
-        let _ = CertificateStore::write_to_file(&pem, &private_key_path, overwrite)?;
-
+        info!("Writing private key to {}", &pkey_path.display());
+        let _ = CertificateStore::write_to_file(&pem, pkey_path, overwrite)?;
         Ok((cert, pkey))
     }
 
+    /// This function will use the supplied arguments to create an Application Instance Certificate
+    /// consisting of a X509v3 certificate and public/private key pair. The cert (including pubkey)
+    /// and private key will be written to disk under the pki path.
+    pub fn create_and_store_application_instance_cert(
+        &self,
+        args: &X509Data,
+        overwrite: bool,
+    ) -> Result<(X509, PrivateKey), String> {
+        CertificateStore::create_certificate_and_key(
+            args,
+            overwrite,
+            &self.own_certificate_path(),
+            &self.own_private_key_path(),
+        )
+    }
+
     /// Validates the cert as trusted and valid. If the cert is unknown, it will be written to
     /// the rejected folder so that the administrator can manually move it to the trusted folder.
     ///
@@ -160,8 +208,19 @@ impl CertificateStore {
     /// A non `Good` status code indicates a failure in the cert or in some action required in
     /// order to validate it.
     ///
-    pub fn validate_or_reject_application_instance_cert(&self, cert: &X509, security_policy: SecurityPolicy, hostname: Option<&str>, application_uri: Option<&str>) -> StatusCode {
-        let result = self.validate_application_instance_cert(cert, security_policy, hostname, application_uri);
+    pub fn validate_or_reject_application_instance_cert(
+        &self,
+        cert: &X509,
+        security_policy: SecurityPolicy,
+        hostname: Option<&str>,
+        application_uri: Option<&str>,
+    ) -> StatusCode {
+        let result = self.validate_application_instance_cert(
+            cert,
+            security_policy,
+            hostname,
+            application_uri,
+        );
         if result.is_bad() {
             match result {
                 StatusCode::BadUnexpectedError | StatusCode::BadSecurityChecksFailed => {
@@ -186,17 +245,19 @@ impl CertificateStore {
             trace!("Cannot find cert on disk");
             false
         } else {
-            let cert2 = CertificateStore::read_cert(cert_path);
-            if cert2.is_err() {
-                trace!("Cannot read cert from disk {:?} - {}", cert_path, cert2.unwrap_err());
-                // No cert2 to compare to
-                false
-            } else {
-                // Compare the buffers
-                trace!("Comparing cert on disk to memory");
-                let der = cert.to_der().unwrap();
-                let der2 = cert2.unwrap().to_der().unwrap();
-                der == der2
+            match CertificateStore::read_cert(cert_path) {
+                Ok(file_der) => {
+                    // Compare the buffers
+                    trace!("Comparing cert on disk to memory");
+                    let der = cert.to_der().unwrap();
+                    let file_der = file_der.to_der().unwrap();
+                    der == file_der
+                }
+                Err(err) => {
+                    trace!("Cannot read cert from disk {:?} - {}", cert_path, err);
+                    // No cert2 to compare to
+                    false
+                }
             }
         }
     }
@@ -211,8 +272,14 @@ impl CertificateStore {
     /// A non `Good` status code indicates a failure in the cert or in some action required in
     /// order to validate it.
     ///
-    pub fn validate_application_instance_cert(&self, cert: &X509, security_policy: SecurityPolicy, hostname: Option<&str>, application_uri: Option<&str>) -> StatusCode {
-        let cert_file_name = CertificateStore::cert_file_name(&cert);
+    pub fn validate_application_instance_cert(
+        &self,
+        cert: &X509,
+        security_policy: SecurityPolicy,
+        hostname: Option<&str>,
+        application_uri: Option<&str>,
+    ) -> StatusCode {
+        let cert_file_name = CertificateStore::cert_file_name(cert);
         debug!("Validating cert with name on disk {}", cert_file_name);
 
         // Look for the cert in the rejected folder. If it's rejected there is no purpose going
@@ -220,12 +287,18 @@ impl CertificateStore {
         {
             let mut cert_path = self.rejected_certs_dir();
             if !cert_path.exists() {
-                error!("Path for rejected certificates {} does not exist", cert_path.display());
+                error!(
+                    "Path for rejected certificates {} does not exist",
+                    cert_path.display()
+                );
                 return StatusCode::BadUnexpectedError;
             }
             cert_path.push(&cert_file_name);
             if cert_path.exists() {
-                warn!("Certificate {} is untrusted because it resides in the rejected directory", cert_file_name);
+                warn!(
+                    "Certificate {} is untrusted because it resides in the rejected directory",
+                    cert_file_name
+                );
                 return StatusCode::BadSecurityChecksFailed;
             }
         }
@@ -236,7 +309,10 @@ impl CertificateStore {
             // Check the trusted folder
             let mut cert_path = self.trusted_certs_dir();
             if !cert_path.exists() {
-                error!("Path for rejected certificates {} does not exist", cert_path.display());
+                error!(
+                    "Path for rejected certificates {} does not exist",
+                    cert_path.display()
+                );
                 return StatusCode::BadUnexpectedError;
             }
             cert_path.push(&cert_file_name);
@@ -248,7 +324,7 @@ impl CertificateStore {
                     // Put the unknown cert into the trusted folder
                     warn!("Certificate {} is unknown but policy will store it into the trusted directory", cert_file_name);
                     let _ = self.store_trusted_cert(cert);
-                    // Note that we drop through and still check the cert for validity
+                // Note that we drop through and still check the cert for validity
                 } else {
                     warn!("Certificate {} is unknown and untrusted so it will be stored in rejected directory", cert_file_name);
                     let _ = self.store_rejected_cert(cert);
@@ -270,19 +346,33 @@ impl CertificateStore {
                 }
                 Ok(key_length) => {
                     if !security_policy.is_valid_keylength(key_length) {
-                        warn!("Certificate {} has an invalid key length {} for the policy {}", cert_file_name, key_length, security_policy);
+                        warn!(
+                            "Certificate {} has an invalid key length {} for the policy {}",
+                            cert_file_name, key_length, security_policy
+                        );
                         return StatusCode::BadSecurityChecksFailed;
                     }
                 }
             }
 
+            if self.skip_verify_certs {
+                debug!(
+                    "Skipping additional verifications for certificate {}",
+                    cert_file_name
+                );
+                return StatusCode::Good;
+            }
+
             // Now inspect the cert not before / after values to ensure its validity
             if self.check_time {
                 use chrono::Utc;
                 let now = Utc::now();
                 let status_code = cert.is_time_valid(&now);
                 if status_code.is_bad() {
-                    warn!("Certificate {} is not valid for now, check start/end timestamps", cert_file_name);
+                    warn!(
+                        "Certificate {} is not valid for now, check start/end timestamps",
+                        cert_file_name
+                    );
                     return status_code;
                 }
             }
@@ -291,7 +381,10 @@ impl CertificateStore {
             if let Some(hostname) = hostname {
                 let status_code = cert.is_hostname_valid(hostname);
                 if status_code.is_bad() {
-                    warn!("Certificate {} does not have a valid hostname", cert_file_name);
+                    warn!(
+                        "Certificate {} does not have a valid hostname",
+                        cert_file_name
+                    );
                     return status_code;
                 }
             }
@@ -300,7 +393,10 @@ impl CertificateStore {
             if let Some(application_uri) = application_uri {
                 let status_code = cert.is_application_uri_valid(application_uri);
                 if status_code.is_bad() {
-                    warn!("Certificate {} does not have a valid application uri", cert_file_name);
+                    warn!(
+                        "Certificate {} does not have a valid application uri",
+                        cert_file_name
+                    );
                     return status_code;
                 }
             }
@@ -318,7 +414,7 @@ impl CertificateStore {
     /// the cert's common name being empty or not
     pub fn cert_file_name(cert: &X509) -> String {
         let prefix = if let Ok(common_name) = cert.common_name() {
-            common_name.trim().to_string().replace("/", "")
+            common_name.trim().to_string().replace('/', "")
         } else {
             String::new()
         };
@@ -339,7 +435,7 @@ impl CertificateStore {
     ///
     pub fn ensure_pki_path(&self) -> Result<(), String> {
         let mut path = self.pki_path.clone();
-        let subdirs = [OWN_CERTIFICATE_DIR, OWN_PRIVATE_KEY_DIR, TRUSTED_CERTS_DIR, REJECTED_CERTS_DIR];
+        let subdirs = [TRUSTED_CERTS_DIR, REJECTED_CERTS_DIR];
         for subdir in &subdirs {
             path.push(subdir);
             CertificateStore::ensure_dir(&path)?;
@@ -362,37 +458,22 @@ impl CertificateStore {
                 Ok(())
             }
         } else {
-            std::fs::create_dir_all(path).map_err(|_| {
-                format!("Cannot make directories for {}", path.display())
-            })
+            std::fs::create_dir_all(path)
+                .map_err(|_| format!("Cannot make directories for {}", path.display()))
         }
     }
 
     /// Get path to application instance certificate
-    pub fn own_cert_path(&self) -> PathBuf {
-        let mut path = self.own_cert_dir();
-        path.push(OWN_CERTIFICATE_NAME);
+    pub fn own_certificate_path(&self) -> PathBuf {
+        let mut path = PathBuf::from(&self.pki_path);
+        path.push(&self.own_certificate_path);
         path
     }
 
     /// Get path to application instance private key
     pub fn own_private_key_path(&self) -> PathBuf {
-        let mut path = self.private_key_dir();
-        path.push(OWN_PRIVATE_KEY_NAME);
-        path
-    }
-
-    /// Get the path to the application instance key dir
-    pub fn private_key_dir(&self) -> PathBuf {
         let mut path = PathBuf::from(&self.pki_path);
-        path.push(OWN_PRIVATE_KEY_DIR);
-        path
-    }
-
-    /// Get the path to the application instance certificate dir
-    pub fn own_cert_dir(&self) -> PathBuf {
-        let mut path = PathBuf::from(&self.pki_path);
-        path.push(OWN_CERTIFICATE_DIR);
+        path.push(&self.own_private_key_path);
         path
     }
 
@@ -419,7 +500,7 @@ impl CertificateStore {
     ///
     pub fn store_rejected_cert(&self, cert: &X509) -> Result<PathBuf, String> {
         // Store the cert in the rejected folder where untrusted certs go
-        let cert_file_name = CertificateStore::cert_file_name(&cert);
+        let cert_file_name = CertificateStore::cert_file_name(cert);
         let mut cert_path = self.rejected_certs_dir();
         cert_path.push(&cert_file_name);
         let _ = CertificateStore::store_cert(cert, &cert_path, true)?;
@@ -434,8 +515,8 @@ impl CertificateStore {
     /// A string description of any failure
     ///
     fn store_trusted_cert(&self, cert: &X509) -> Result<PathBuf, String> {
-        // Store the cert in the rejected folder where untrusted certs go
-        let cert_file_name = CertificateStore::cert_file_name(&cert);
+        // Store the cert in the trusted folder where trusted certs go
+        let cert_file_name = CertificateStore::cert_file_name(cert);
         let mut cert_path = self.trusted_certs_dir();
         cert_path.push(&cert_file_name);
         let _ = CertificateStore::store_cert(cert, &cert_path, true)?;
@@ -451,10 +532,10 @@ impl CertificateStore {
     fn store_cert(cert: &X509, path: &Path, overwrite: bool) -> Result<usize, String> {
         let der = cert.to_der().unwrap();
         info!("Writing X509 cert to {}", path.display());
-        CertificateStore::write_to_file(&der, &path, overwrite)
+        CertificateStore::write_to_file(&der, path, overwrite)
     }
 
-    /// Reads an X509 certificate in .def format from disk
+    /// Reads an X509 certificate in .def or .pem format from disk
     ///
     /// # Errors
     ///
@@ -470,30 +551,27 @@ impl CertificateStore {
         let mut cert = Vec::new();
         let bytes_read = file.read_to_end(&mut cert);
         if bytes_read.is_err() {
-            return Err(format!("Could not read bytes from cert file {}", path.display()));
+            return Err(format!(
+                "Could not read bytes from cert file {}",
+                path.display()
+            ));
         }
 
-        let cert = x509::X509::from_der(&cert);
+        let cert = match path.extension() {
+            Some(v) if v == "der" => x509::X509::from_der(&cert),
+            Some(v) if v == "pem" => x509::X509::from_pem(&cert),
+            _ => return Err("Only .der and .pem certificates are supported".to_string()),
+        };
         if cert.is_err() {
-            return Err(format!("Could not read cert from cert file {}", path.display()));
+            return Err(format!(
+                "Could not read cert from cert file {}",
+                path.display()
+            ));
         }
 
         Ok(X509::from(cert.unwrap()))
     }
 
-    /// Makes a path
-    ///
-    /// # Errors
-    ///
-    /// A string description of any failure
-    ///
-    fn make_and_ensure_file_path(path: &Path, file_name: &str) -> Result<PathBuf, String> {
-        let mut path = PathBuf::from(&path);
-        CertificateStore::ensure_dir(&path)?;
-        path.push(file_name);
-        Ok(path)
-    }
-
     /// Writes bytes to file and returns the size written, or an error reason for failure.
     ///
     /// # Errors
@@ -504,13 +582,15 @@ impl CertificateStore {
         if !overwrite && file_path.exists() {
             Err(format!("File {} already exists and will not be overwritten. Enable overwrite to disable this safeguard.", file_path.display()))
         } else {
+            if let Some(parent) = file_path.parent() {
+                CertificateStore::ensure_dir(parent)?;
+            }
             match File::create(file_path) {
-                Ok(mut file) => file.write(bytes)
-                    .map_err(|_| {
-                        format!("Could not write bytes to file {}", file_path.display())
-                    }),
-                Err(_) => Err(format!("Could not create file {}", file_path.display()))
+                Ok(mut file) => file
+                    .write(bytes)
+                    .map_err(|_| format!("Could not write bytes to file {}", file_path.display())),
+                Err(_) => Err(format!("Could not create file {}", file_path.display())),
             }
         }
     }
-}
\ No newline at end of file
+}
