@@ -1,25 +1,21 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 
 //! Security policy is the symmetric, asymmetric encryption / decryption + signing / verification
 //! algorithms to use and enforce for the current session.
-use opcua_types::{
-    ByteString,
-    constants,
-    status_code::StatusCode,
-};
-use openssl::hash as openssl_hash;
 use std::fmt;
 use std::str::FromStr;
 
+use openssl::hash as openssl_hash;
+
+use opcua_types::{constants, status_code::StatusCode, ByteString};
+
 use crate::{
     aeskey::AesKey,
     hash,
     pkey::{KeySize, PrivateKey, PublicKey, RsaPadding},
-    random,
-    SHA1_SIZE,
-    SHA256_SIZE,
+    random, SHA1_SIZE, SHA256_SIZE,
 };
 
 // These are constants that govern the different encryption / signing modes for OPC UA. In some
@@ -44,7 +40,8 @@ mod aes_128_sha_256_rsa_oaep {
     use crate::algorithms::*;
 
     pub const SECURITY_POLICY: &str = "Aes128-Sha256-RsaOaep";
-    pub const SECURITY_POLICY_URI: &str = "http://opcfoundation.org/UA/SecurityPolicy#Aes128_Sha256_RsaOaep";
+    pub const SECURITY_POLICY_URI: &str =
+        "http://opcfoundation.org/UA/SecurityPolicy#Aes128_Sha256_RsaOaep";
 
     pub const SYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_HMAC_SHA256;
     pub const ASYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_RSA_SHA256;
@@ -71,7 +68,8 @@ mod aes_256_sha_256_rsa_pss {
     use crate::algorithms::*;
 
     pub const SECURITY_POLICY: &str = "Aes256-Sha256-RsaPss";
-    pub const SECURITY_POLICY_URI: &str = "http://opcfoundation.org/UA/SecurityPolicy#Aes256_Sha256_RsaPss";
+    pub const SECURITY_POLICY_URI: &str =
+        "http://opcfoundation.org/UA/SecurityPolicy#Aes256_Sha256_RsaPss";
 
     pub const SYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_HMAC_SHA256;
     pub const ASYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_RSA_PSS_SHA2_256;
@@ -98,7 +96,8 @@ mod basic_256_sha_256 {
     use crate::algorithms::*;
 
     pub const SECURITY_POLICY: &str = "Basic256Sha256";
-    pub const SECURITY_POLICY_URI: &str = "http://opcfoundation.org/UA/SecurityPolicy#Basic256Sha256";
+    pub const SECURITY_POLICY_URI: &str =
+        "http://opcfoundation.org/UA/SecurityPolicy#Basic256Sha256";
 
     pub const SYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_HMAC_SHA256;
     pub const ASYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_RSA_SHA256;
@@ -124,7 +123,8 @@ mod basic_128_rsa_15 {
     use crate::algorithms::*;
 
     pub const SECURITY_POLICY: &str = "Basic128Rsa15";
-    pub const SECURITY_POLICY_URI: &str = "http://opcfoundation.org/UA/SecurityPolicy#Basic128Rsa15";
+    pub const SECURITY_POLICY_URI: &str =
+        "http://opcfoundation.org/UA/SecurityPolicy#Basic128Rsa15";
 
     pub const SYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_HMAC_SHA1;
     pub const ASYMMETRIC_SIGNATURE_ALGORITHM: &str = DSIG_RSA_SHA1;
@@ -183,12 +183,20 @@ impl FromStr for SecurityPolicy {
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         Ok(match s {
-            constants::SECURITY_POLICY_NONE | constants::SECURITY_POLICY_NONE_URI => SecurityPolicy::None,
-            basic_128_rsa_15::SECURITY_POLICY | basic_128_rsa_15::SECURITY_POLICY_URI => SecurityPolicy::Basic128Rsa15,
+            constants::SECURITY_POLICY_NONE | constants::SECURITY_POLICY_NONE_URI => {
+                SecurityPolicy::None
+            }
+            basic_128_rsa_15::SECURITY_POLICY | basic_128_rsa_15::SECURITY_POLICY_URI => {
+                SecurityPolicy::Basic128Rsa15
+            }
             basic_256::SECURITY_POLICY | basic_256::SECURITY_POLICY_URI => SecurityPolicy::Basic256,
-            basic_256_sha_256::SECURITY_POLICY | basic_256_sha_256::SECURITY_POLICY_URI => SecurityPolicy::Basic256Sha256,
-            aes_128_sha_256_rsa_oaep::SECURITY_POLICY | aes_128_sha_256_rsa_oaep::SECURITY_POLICY_URI => SecurityPolicy::Aes128Sha256RsaOaep,
-            aes_256_sha_256_rsa_pss::SECURITY_POLICY | aes_256_sha_256_rsa_pss::SECURITY_POLICY_URI => SecurityPolicy::Aes256Sha256RsaPss,
+            basic_256_sha_256::SECURITY_POLICY | basic_256_sha_256::SECURITY_POLICY_URI => {
+                SecurityPolicy::Basic256Sha256
+            }
+            aes_128_sha_256_rsa_oaep::SECURITY_POLICY
+            | aes_128_sha_256_rsa_oaep::SECURITY_POLICY_URI => SecurityPolicy::Aes128Sha256RsaOaep,
+            aes_256_sha_256_rsa_pss::SECURITY_POLICY
+            | aes_256_sha_256_rsa_pss::SECURITY_POLICY_URI => SecurityPolicy::Aes256Sha256RsaPss,
             _ => {
                 error!("Specified security policy \"{}\" is not recognized", s);
                 SecurityPolicy::Unknown
@@ -220,21 +228,24 @@ impl SecurityPolicy {
 
     /// Returns true if the security policy is supported. It might be recognized but be unsupported by the implementation
     pub fn is_supported(&self) -> bool {
-        match self {
-            SecurityPolicy::None | SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 |
-            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep => true,
-            // | SecurityPolicy::Aes256Sha256RsaPss
-            _ => false
-        }
+        matches!(
+            self,
+            SecurityPolicy::None
+                | SecurityPolicy::Basic128Rsa15
+                | SecurityPolicy::Basic256
+                | SecurityPolicy::Basic256Sha256
+                | SecurityPolicy::Aes128Sha256RsaOaep
+                | SecurityPolicy::Aes256Sha256RsaPss
+        )
     }
 
     /// Returns true if the security policy has been deprecated by the OPC UA specification
     pub fn is_deprecated(&self) -> bool {
-        match self {
-            // Since 1.04 because SHA-1 is no longer considered safe
-            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => true,
-            _ => false
-        }
+        // Since 1.04 because SHA-1 is no longer considered safe
+        matches!(
+            self,
+            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256
+        )
     }
 
     pub fn to_str(&self) -> &'static str {
@@ -256,8 +267,12 @@ impl SecurityPolicy {
             SecurityPolicy::Basic128Rsa15 => basic_128_rsa_15::ASYMMETRIC_ENCRYPTION_ALGORITHM,
             SecurityPolicy::Basic256 => basic_256::ASYMMETRIC_ENCRYPTION_ALGORITHM,
             SecurityPolicy::Basic256Sha256 => basic_256_sha_256::ASYMMETRIC_ENCRYPTION_ALGORITHM,
-            SecurityPolicy::Aes128Sha256RsaOaep => aes_128_sha_256_rsa_oaep::ASYMMETRIC_ENCRYPTION_ALGORITHM,
-            SecurityPolicy::Aes256Sha256RsaPss => aes_256_sha_256_rsa_pss::ASYMMETRIC_ENCRYPTION_ALGORITHM,
+            SecurityPolicy::Aes128Sha256RsaOaep => {
+                aes_128_sha_256_rsa_oaep::ASYMMETRIC_ENCRYPTION_ALGORITHM
+            }
+            SecurityPolicy::Aes256Sha256RsaPss => {
+                aes_256_sha_256_rsa_pss::ASYMMETRIC_ENCRYPTION_ALGORITHM
+            }
             _ => {
                 panic!("Invalid policy");
             }
@@ -269,8 +284,12 @@ impl SecurityPolicy {
             SecurityPolicy::Basic128Rsa15 => basic_128_rsa_15::ASYMMETRIC_SIGNATURE_ALGORITHM,
             SecurityPolicy::Basic256 => basic_256::ASYMMETRIC_SIGNATURE_ALGORITHM,
             SecurityPolicy::Basic256Sha256 => basic_256_sha_256::ASYMMETRIC_SIGNATURE_ALGORITHM,
-            SecurityPolicy::Aes128Sha256RsaOaep => aes_128_sha_256_rsa_oaep::ASYMMETRIC_SIGNATURE_ALGORITHM,
-            SecurityPolicy::Aes256Sha256RsaPss => aes_256_sha_256_rsa_pss::ASYMMETRIC_SIGNATURE_ALGORITHM,
+            SecurityPolicy::Aes128Sha256RsaOaep => {
+                aes_128_sha_256_rsa_oaep::ASYMMETRIC_SIGNATURE_ALGORITHM
+            }
+            SecurityPolicy::Aes256Sha256RsaPss => {
+                aes_256_sha_256_rsa_pss::ASYMMETRIC_SIGNATURE_ALGORITHM
+            }
             _ => {
                 panic!("Invalid policy");
             }
@@ -282,8 +301,12 @@ impl SecurityPolicy {
             SecurityPolicy::Basic128Rsa15 => basic_128_rsa_15::SYMMETRIC_SIGNATURE_ALGORITHM,
             SecurityPolicy::Basic256 => basic_256::SYMMETRIC_SIGNATURE_ALGORITHM,
             SecurityPolicy::Basic256Sha256 => basic_256_sha_256::SYMMETRIC_SIGNATURE_ALGORITHM,
-            SecurityPolicy::Aes128Sha256RsaOaep => aes_128_sha_256_rsa_oaep::SYMMETRIC_SIGNATURE_ALGORITHM,
-            SecurityPolicy::Aes256Sha256RsaPss => aes_256_sha_256_rsa_pss::SYMMETRIC_SIGNATURE_ALGORITHM,
+            SecurityPolicy::Aes128Sha256RsaOaep => {
+                aes_128_sha_256_rsa_oaep::SYMMETRIC_SIGNATURE_ALGORITHM
+            }
+            SecurityPolicy::Aes256Sha256RsaPss => {
+                aes_256_sha_256_rsa_pss::SYMMETRIC_SIGNATURE_ALGORITHM
+            }
             _ => {
                 panic!("Invalid policy");
             }
@@ -293,8 +316,11 @@ impl SecurityPolicy {
     // Plaintext block size in bytes
     pub fn plain_block_size(&self) -> usize {
         match self {
-            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 | SecurityPolicy::Basic256Sha256 |
-            SecurityPolicy::Aes128Sha256RsaOaep | SecurityPolicy::Aes256Sha256RsaPss => 16,
+            SecurityPolicy::Basic128Rsa15
+            | SecurityPolicy::Basic256
+            | SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes128Sha256RsaOaep
+            | SecurityPolicy::Aes256Sha256RsaPss => 16,
             _ => {
                 panic!("Invalid policy");
             }
@@ -306,8 +332,9 @@ impl SecurityPolicy {
         match self {
             SecurityPolicy::None => 0,
             SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => SHA1_SIZE,
-            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep |
-            SecurityPolicy::Aes256Sha256RsaPss => SHA256_SIZE,
+            SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes128Sha256RsaOaep
+            | SecurityPolicy::Aes256Sha256RsaPss => SHA256_SIZE,
             _ => {
                 panic!("Invalid policy");
             }
@@ -320,8 +347,12 @@ impl SecurityPolicy {
             SecurityPolicy::Basic128Rsa15 => basic_128_rsa_15::DERIVED_SIGNATURE_KEY_LENGTH,
             SecurityPolicy::Basic256 => basic_256::DERIVED_SIGNATURE_KEY_LENGTH,
             SecurityPolicy::Basic256Sha256 => basic_256_sha_256::DERIVED_SIGNATURE_KEY_LENGTH,
-            SecurityPolicy::Aes128Sha256RsaOaep => aes_128_sha_256_rsa_oaep::DERIVED_SIGNATURE_KEY_LENGTH,
-            SecurityPolicy::Aes256Sha256RsaPss => aes_256_sha_256_rsa_pss::DERIVED_SIGNATURE_KEY_LENGTH,
+            SecurityPolicy::Aes128Sha256RsaOaep => {
+                aes_128_sha_256_rsa_oaep::DERIVED_SIGNATURE_KEY_LENGTH
+            }
+            SecurityPolicy::Aes256Sha256RsaPss => {
+                aes_256_sha_256_rsa_pss::DERIVED_SIGNATURE_KEY_LENGTH
+            }
             _ => {
                 panic!("Invalid policy");
             }
@@ -360,9 +391,11 @@ impl SecurityPolicy {
     pub fn secure_channel_nonce_length(&self) -> usize {
         match self {
             SecurityPolicy::Basic128Rsa15 => 16,
-            SecurityPolicy::Basic256 | SecurityPolicy::Basic256Sha256 |
-            SecurityPolicy::Aes128Sha256RsaOaep | SecurityPolicy::Aes256Sha256RsaPss => 32,
-            _ => panic!("")
+            SecurityPolicy::Basic256
+            | SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes128Sha256RsaOaep
+            | SecurityPolicy::Aes256Sha256RsaPss => 32,
+            _ => panic!(""),
         }
     }
 
@@ -375,7 +408,10 @@ impl SecurityPolicy {
             aes_128_sha_256_rsa_oaep::SECURITY_POLICY_URI => SecurityPolicy::Aes128Sha256RsaOaep,
             aes_256_sha_256_rsa_pss::SECURITY_POLICY_URI => SecurityPolicy::Aes256Sha256RsaPss,
             _ => {
-                error!("Specified security policy uri \"{}\" is not recognized", uri);
+                error!(
+                    "Specified security policy uri \"{}\" is not recognized",
+                    uri
+                );
                 SecurityPolicy::Unknown
             }
         }
@@ -386,9 +422,12 @@ impl SecurityPolicy {
     fn prf(&self, secret: &[u8], seed: &[u8], length: usize, offset: usize) -> Vec<u8> {
         // P_SHA1 or P_SHA256
         let message_digest = match self {
-            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => openssl_hash::MessageDigest::sha1(),
-            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep |
-            SecurityPolicy::Aes256Sha256RsaPss => openssl_hash::MessageDigest::sha256(),
+            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => {
+                openssl_hash::MessageDigest::sha1()
+            }
+            SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes128Sha256RsaOaep
+            | SecurityPolicy::Aes256Sha256RsaPss => openssl_hash::MessageDigest::sha256(),
             _ => {
                 panic!("Invalid policy");
             }
@@ -432,12 +471,18 @@ impl SecurityPolicy {
     /// The Client keys are used to secure Messages sent by the Client. The Server keys
     /// are used to secure Messages sent by the Server.
     ///
-    pub fn make_secure_channel_keys(&self, secret: &[u8], seed: &[u8]) -> (Vec<u8>, AesKey, Vec<u8>) {
+    pub fn make_secure_channel_keys(
+        &self,
+        secret: &[u8],
+        seed: &[u8],
+    ) -> (Vec<u8>, AesKey, Vec<u8>) {
         // Work out the length of stuff
         let signing_key_length = self.derived_signature_key_size();
         let (encrypting_key_length, encrypting_block_size) = match self {
             SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Aes128Sha256RsaOaep => (16, 16),
-            SecurityPolicy::Basic256 | SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes256Sha256RsaPss => (32, 16),
+            SecurityPolicy::Basic256
+            | SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes256Sha256RsaPss => (32, 16),
             _ => {
                 panic!("Invalid policy");
             }
@@ -446,18 +491,32 @@ impl SecurityPolicy {
         let signing_key = self.prf(secret, seed, signing_key_length, 0);
         let encrypting_key = self.prf(secret, seed, encrypting_key_length, signing_key_length);
         let encrypting_key = AesKey::new(*self, &encrypting_key);
-        let iv = self.prf(secret, seed, encrypting_block_size, signing_key_length + encrypting_key_length);
+        let iv = self.prf(
+            secret,
+            seed,
+            encrypting_block_size,
+            signing_key_length + encrypting_key_length,
+        );
 
         (signing_key, encrypting_key, iv)
     }
 
     /// Produce a signature of the data using an asymmetric key. Stores the signature in the supplied
     /// `signature` buffer. Returns the size of the signature within that buffer.
-    pub fn asymmetric_sign(&self, signing_key: &PrivateKey, data: &[u8], signature: &mut [u8]) -> Result<usize, StatusCode> {
+    pub fn asymmetric_sign(
+        &self,
+        signing_key: &PrivateKey,
+        data: &[u8],
+        signature: &mut [u8],
+    ) -> Result<usize, StatusCode> {
         let result = match self {
-            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => signing_key.sign_hmac_sha1(data, signature)?,
-            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep => signing_key.sign_hmac_sha256(data, signature)?,
-            SecurityPolicy::Aes256Sha256RsaPss => signing_key.sign_hmac_sha256_pss(data, signature)?,
+            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => {
+                signing_key.sign_sha1(data, signature)?
+            }
+            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep => {
+                signing_key.sign_sha256(data, signature)?
+            }
+            SecurityPolicy::Aes256Sha256RsaPss => signing_key.sign_sha256_pss(data, signature)?,
             _ => {
                 panic!("Invalid policy");
             }
@@ -468,12 +527,24 @@ impl SecurityPolicy {
     /// Verifies a signature of the data using an asymmetric key. In a debugging scenario, the
     /// signing key can also be supplied so that the supplied signature can be compared to a freshly
     /// generated signature.
-    pub fn asymmetric_verify_signature(&self, verification_key: &PublicKey, data: &[u8], signature: &[u8], their_private_key: Option<PrivateKey>) -> Result<(), StatusCode> {
+    pub fn asymmetric_verify_signature(
+        &self,
+        verification_key: &PublicKey,
+        data: &[u8],
+        signature: &[u8],
+        their_private_key: Option<PrivateKey>,
+    ) -> Result<(), StatusCode> {
         // Asymmetric verify signature against supplied certificate
         let result = match self {
-            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => verification_key.verify_hmac_sha1(data, signature)?,
-            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep => verification_key.verify_hmac_sha256(data, signature)?,
-            SecurityPolicy::Aes256Sha256RsaPss => verification_key.verify_hmac_sha256_pss(data, signature)?,
+            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => {
+                verification_key.verify_sha1(data, signature)?
+            }
+            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep => {
+                verification_key.verify_sha256(data, signature)?
+            }
+            SecurityPolicy::Aes256Sha256RsaPss => {
+                verification_key.verify_sha256_pss(data, signature)?
+            }
             _ => {
                 panic!("Invalid policy");
             }
@@ -488,7 +559,10 @@ impl SecurityPolicy {
                 // Calculate the signature using their key, see what we were expecting versus theirs
                 let mut their_signature = vec![0u8; their_key.size()];
                 self.asymmetric_sign(&their_key, data, their_signature.as_mut_slice())?;
-                trace!("Using their_key, signature should be {:?}", &their_signature);
+                trace!(
+                    "Using their_key, signature should be {:?}",
+                    &their_signature
+                );
             }
             Err(StatusCode::BadSecurityChecksFailed)
         }
@@ -498,9 +572,12 @@ impl SecurityPolicy {
     /// and decryption.
     pub fn asymmetric_encryption_padding(&self) -> RsaPadding {
         match self {
-            SecurityPolicy::Basic128Rsa15 => RsaPadding::PKCS1,
-            SecurityPolicy::Basic256 | SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep |
-            SecurityPolicy::Aes256Sha256RsaPss => RsaPadding::OAEP,
+            SecurityPolicy::Basic128Rsa15 => RsaPadding::Pkcs1,
+            SecurityPolicy::Basic256
+            | SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes128Sha256RsaOaep => RsaPadding::OaepSha1,
+            // PSS uses OAEP-SHA256 for encryption, but PSS for signing
+            SecurityPolicy::Aes256Sha256RsaPss => RsaPadding::OaepSha256,
             _ => {
                 panic!("Security policy is not supported, shouldn't have gotten here");
             }
@@ -509,18 +586,30 @@ impl SecurityPolicy {
 
     /// Encrypts a message using the supplied encryption key, returns the encrypted size. Destination
     /// buffer must be large enough to hold encrypted bytes including any padding.
-    pub fn asymmetric_encrypt(&self, encryption_key: &PublicKey, src: &[u8], dst: &mut [u8]) -> Result<usize, StatusCode> {
+    pub fn asymmetric_encrypt(
+        &self,
+        encryption_key: &PublicKey,
+        src: &[u8],
+        dst: &mut [u8],
+    ) -> Result<usize, StatusCode> {
         let padding = self.asymmetric_encryption_padding();
-        encryption_key.public_encrypt(src, dst, padding)
+        encryption_key
+            .public_encrypt(src, dst, padding)
             .map_err(|_| StatusCode::BadUnexpectedError)
     }
 
     /// Decrypts a message whose thumbprint matches the x509 cert and private key pair.
     ///
     /// Returns the number of decrypted bytes
-    pub fn asymmetric_decrypt(&self, decryption_key: &PrivateKey, src: &[u8], dst: &mut [u8]) -> Result<usize, StatusCode> {
+    pub fn asymmetric_decrypt(
+        &self,
+        decryption_key: &PrivateKey,
+        src: &[u8],
+        dst: &mut [u8],
+    ) -> Result<usize, StatusCode> {
         let padding = self.asymmetric_encryption_padding();
-        decryption_key.private_decrypt(src, dst, padding)
+        decryption_key
+            .private_decrypt(src, dst, padding)
             .map_err(|_| {
                 error!("Asymmetric decryption failed");
                 StatusCode::BadSecurityChecksFailed
@@ -529,11 +618,24 @@ impl SecurityPolicy {
 
     /// Produce a signature of some data using the supplied symmetric key. Signing algorithm is determined
     /// by the security policy. Signature is stored in the supplied `signature` argument.
-    pub fn symmetric_sign(&self, key: &[u8], data: &[u8], signature: &mut [u8]) -> Result<(), StatusCode> {
-        trace!("Producing signature for {} bytes of data into signature of {} bytes", data.len(), signature.len());
+    pub fn symmetric_sign(
+        &self,
+        key: &[u8],
+        data: &[u8],
+        signature: &mut [u8],
+    ) -> Result<(), StatusCode> {
+        trace!(
+            "Producing signature for {} bytes of data into signature of {} bytes",
+            data.len(),
+            signature.len()
+        );
         match self {
-            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => hash::hmac_sha1(key, data, signature),
-            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep | SecurityPolicy::Aes256Sha256RsaPss => hash::hmac_sha256(key, data, signature),
+            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => {
+                hash::hmac_sha1(key, data, signature)
+            }
+            SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes128Sha256RsaOaep
+            | SecurityPolicy::Aes256Sha256RsaPss => hash::hmac_sha256(key, data, signature),
             _ => {
                 panic!("Unsupported policy")
             }
@@ -541,11 +643,20 @@ impl SecurityPolicy {
     }
 
     /// Verify the signature of a data block using the supplied symmetric key.
-    pub fn symmetric_verify_signature(&self, key: &[u8], data: &[u8], signature: &[u8]) -> Result<bool, StatusCode> {
+    pub fn symmetric_verify_signature(
+        &self,
+        key: &[u8],
+        data: &[u8],
+        signature: &[u8],
+    ) -> Result<bool, StatusCode> {
         // Verify the signature using SHA-1 / SHA-256 HMAC
         let verified = match self {
-            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => hash::verify_hmac_sha1(key, data, signature),
-            SecurityPolicy::Basic256Sha256 | SecurityPolicy::Aes128Sha256RsaOaep | SecurityPolicy::Aes256Sha256RsaPss => hash::verify_hmac_sha256(key, data, signature),
+            SecurityPolicy::Basic128Rsa15 | SecurityPolicy::Basic256 => {
+                hash::verify_hmac_sha1(key, data, signature)
+            }
+            SecurityPolicy::Basic256Sha256
+            | SecurityPolicy::Aes128Sha256RsaOaep
+            | SecurityPolicy::Aes256Sha256RsaPss => hash::verify_hmac_sha256(key, data, signature),
             _ => {
                 panic!("Unsupported policy")
             }
@@ -559,12 +670,24 @@ impl SecurityPolicy {
     }
 
     /// Encrypt the supplied data using the supplied key storing the result in the destination.
-    pub fn symmetric_encrypt(&self, key: &AesKey, iv: &[u8], src: &[u8], dst: &mut [u8]) -> Result<usize, StatusCode> {
+    pub fn symmetric_encrypt(
+        &self,
+        key: &AesKey,
+        iv: &[u8],
+        src: &[u8],
+        dst: &mut [u8],
+    ) -> Result<usize, StatusCode> {
         key.encrypt(src, iv, dst)
     }
 
     /// Decrypts the supplied data using the supplied key storing the result in the destination.
-    pub fn symmetric_decrypt(&self, key: &AesKey, iv: &[u8], src: &[u8], dst: &mut [u8]) -> Result<usize, StatusCode> {
+    pub fn symmetric_decrypt(
+        &self,
+        key: &AesKey,
+        iv: &[u8],
+        src: &[u8],
+        dst: &mut [u8],
+    ) -> Result<usize, StatusCode> {
         key.decrypt(src, iv, dst)
     }
 }
