@@ -1,12 +1,12 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 
 // X509 certificate wrapper.
 
 use std::{
     self,
-    fmt::{Debug, Formatter},
+    fmt::{self, Debug, Formatter},
     net::{Ipv4Addr, Ipv6Addr},
     result::Result,
 };
@@ -21,7 +21,7 @@ use openssl::{
     x509::{self, extension::*},
 };
 
-use opcua_types::{ByteString, service_types::ApplicationDescription, status_code::StatusCode};
+use opcua_types::{service_types::ApplicationDescription, status_code::StatusCode, ByteString};
 
 use crate::{
     hostname,
@@ -101,7 +101,12 @@ impl X509Data {
     }
 
     /// Creates a list of uri + DNS hostnames using the supplied arguments
-    pub fn alt_host_names(application_uri: &str, addresses: Option<Vec<String>>, add_localhost: bool, add_computer_name: bool) -> Vec<String> {
+    pub fn alt_host_names(
+        application_uri: &str,
+        addresses: Option<Vec<String>>,
+        add_localhost: bool,
+        add_computer_name: bool,
+    ) -> Vec<String> {
         // The first name is the application uri
         let mut result = vec![application_uri.to_string()];
 
@@ -142,6 +147,17 @@ impl X509Data {
     }
 }
 
+#[derive(Debug)]
+pub struct X509Error;
+
+impl fmt::Display for X509Error {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "X509Error")
+    }
+}
+
+impl std::error::Error for X509Error {}
+
 /// This is a wrapper around the `OpenSSL` `X509` cert
 #[derive(Clone)]
 pub struct X509 {
@@ -163,12 +179,11 @@ impl From<x509::X509> for X509 {
 }
 
 impl X509 {
-    pub fn from_der(der: &[u8]) -> Result<Self, ()> {
-        x509::X509::from_der(der)
-            .map(|value| X509::from(value))
-            .map_err(|_| {
-                error!("Cannot produce an x509 cert from the data supplied");
-            })
+    pub fn from_der(der: &[u8]) -> Result<Self, X509Error> {
+        x509::X509::from_der(der).map(X509::from).map_err(|_| {
+            error!("Cannot produce an x509 cert from the data supplied");
+            X509Error
+        })
     }
 
     /// Creates a self-signed X509v3 certificate and public/private key from the supplied creation args.
@@ -183,14 +198,14 @@ impl X509 {
     /// hostnames / ip addresses that the host runs on.
     pub fn cert_and_pkey(x509_data: &X509Data) -> Result<(Self, PrivateKey), String> {
         // Create a key pair
-        let rsa = Rsa::generate(x509_data.key_size)
-            .map_err(|err| {
-                format!("Cannot create key pair check error {} and key size {}", err.to_string(), x509_data.key_size)
-            })?;
+        let rsa = Rsa::generate(x509_data.key_size).map_err(|err| {
+            format!(
+                "Cannot create key pair check error {} and key size {}",
+                err, x509_data.key_size
+            )
+        })?;
         let pkey = pkey::PKey::from_rsa(rsa)
-            .map_err(|err| {
-                format!("Cannot create key pair check error {}", err.to_string())
-            })?;
+            .map_err(|err| format!("Cannot create key pair check error {}", err))?;
         let pkey = PrivateKey::wrap_private_key(pkey);
 
         // Create an X509 cert to hold the public key
@@ -206,11 +221,14 @@ impl X509 {
         let issuer_name = {
             let mut name = x509::X509NameBuilder::new().unwrap();
             // Common name
-            name.append_entry_by_text("CN", &x509_data.common_name).unwrap();
+            name.append_entry_by_text("CN", &x509_data.common_name)
+                .unwrap();
             // Organization
-            name.append_entry_by_text("O", &x509_data.organization).unwrap();
+            name.append_entry_by_text("O", &x509_data.organization)
+                .unwrap();
             // Organizational Unit
-            name.append_entry_by_text("OU", &x509_data.organizational_unit).unwrap();
+            name.append_entry_by_text("OU", &x509_data.organizational_unit)
+                .unwrap();
             // Country
             name.append_entry_by_text("C", &x509_data.country).unwrap();
             // State
@@ -222,21 +240,28 @@ impl X509 {
         let _ = builder.set_issuer_name(&issuer_name);
 
         // For Application Instance Certificate specifies how cert may be used
-        let key_usage = KeyUsage::new().
-            digital_signature().
-            non_repudiation().
-            key_encipherment().
-            data_encipherment().
-            key_cert_sign().
-            build().unwrap();
+        let key_usage = KeyUsage::new()
+            .digital_signature()
+            .non_repudiation()
+            .key_encipherment()
+            .data_encipherment()
+            .key_cert_sign()
+            .build()
+            .unwrap();
         let _ = builder.append_extension(key_usage);
-        let extended_key_usage = ExtendedKeyUsage::new().
-            client_auth().
-            server_auth().build().unwrap();
+        let extended_key_usage = ExtendedKeyUsage::new()
+            .client_auth()
+            .server_auth()
+            .build()
+            .unwrap();
         let _ = builder.append_extension(extended_key_usage);
 
-        builder.set_not_before(&Asn1Time::days_from_now(0).unwrap()).unwrap();
-        builder.set_not_after(&Asn1Time::days_from_now(x509_data.certificate_duration_days).unwrap()).unwrap();
+        builder
+            .set_not_before(&Asn1Time::days_from_now(0).unwrap())
+            .unwrap();
+        builder
+            .set_not_after(&Asn1Time::days_from_now(x509_data.certificate_duration_days).unwrap())
+            .unwrap();
         builder.set_pubkey(&pkey.value).unwrap();
 
         // Random serial number
@@ -254,24 +279,29 @@ impl X509 {
         if !x509_data.alt_host_names.is_empty() {
             let subject_alternative_name = {
                 let mut subject_alternative_name = SubjectAlternativeName::new();
-                x509_data.alt_host_names.iter().enumerate().for_each(|(i, alt_host_name)| {
-                    if !alt_host_name.is_empty() {
-                        if i == 0 {
-                            // The first entry is the application uri
-                            subject_alternative_name.uri(alt_host_name);
-                        } else if let Ok(_) = alt_host_name.parse::<Ipv4Addr>() {
-                            // Treat this as an IPv4 address
-                            subject_alternative_name.ip(alt_host_name);
-                        } else if let Ok(_) = alt_host_name.parse::<Ipv6Addr>() {
-                            // Treat this as an IPv6 address
-                            subject_alternative_name.ip(alt_host_name);
-                        } else {
-                            // Treat this as a DNS entries
-                            subject_alternative_name.dns(alt_host_name);
+                x509_data
+                    .alt_host_names
+                    .iter()
+                    .enumerate()
+                    .for_each(|(i, alt_host_name)| {
+                        if !alt_host_name.is_empty() {
+                            if i == 0 {
+                                // The first entry is the application uri
+                                subject_alternative_name.uri(alt_host_name);
+                            } else if alt_host_name.parse::<Ipv4Addr>().is_ok()
+                                || alt_host_name.parse::<Ipv6Addr>().is_ok()
+                            {
+                                // Treat this as an IPv4/IPv6 address
+                                subject_alternative_name.ip(alt_host_name);
+                            } else {
+                                // Treat this as a DNS entry
+                                subject_alternative_name.dns(alt_host_name);
+                            }
                         }
-                    }
-                });
-                subject_alternative_name.build(&builder.x509v3_context(None, None)).unwrap()
+                    });
+                subject_alternative_name
+                    .build(&builder.x509v3_context(None, None))
+                    .unwrap()
             };
             builder.append_extension(subject_alternative_name).unwrap();
         }
@@ -286,7 +316,7 @@ impl X509 {
         if data.is_null() {
             error!("Cannot make certificate from null bytestring");
             Err(StatusCode::BadCertificateInvalid)
-        } else if let Ok(cert) = x509::X509::from_der(&data.value.as_ref().unwrap()) {
+        } else if let Ok(cert) = x509::X509::from_der(data.value.as_ref().unwrap()) {
             Ok(X509::from(cert))
         } else {
             error!("Cannot make certificate, does bytestring contain .der?");
@@ -301,8 +331,9 @@ impl X509 {
     }
 
     pub fn public_key(&self) -> Result<PublicKey, StatusCode> {
-        self.value.public_key()
-            .map(|pkey| PublicKey::wrap_public_key(pkey))
+        self.value
+            .public_key()
+            .map(PublicKey::wrap_public_key)
             .map_err(|_| {
                 error!("Cannot obtain public key from certificate");
                 StatusCode::BadCertificateInvalid
@@ -310,12 +341,12 @@ impl X509 {
     }
 
     /// Returns the key length in bits (if possible)
-    pub fn key_length(&self) -> Result<usize, ()> {
-        let pub_key = self.value.public_key().map_err(|_| ())?;
+    pub fn key_length(&self) -> Result<usize, X509Error> {
+        let pub_key = self.value.public_key().map_err(|_| X509Error)?;
         Ok(pub_key.size() * 8)
     }
 
-    fn get_subject_entry(&self, nid: Nid) -> Result<String, ()> {
+    fn get_subject_entry(&self, nid: Nid) -> Result<String, X509Error> {
         let subject_name = self.value.subject_name();
         let mut entries = subject_name.entries_by_nid(nid);
         if let Some(entry) = entries.next() {
@@ -325,17 +356,19 @@ impl X509 {
                 // Value is an OpensslString type here so it has to be converted
                 Ok(value.deref().to_string())
             } else {
-                Err(())
+                Err(X509Error)
             }
         } else {
-            Err(())
+            Err(X509Error)
         }
     }
 
     // Produces a string such as "CN=foo/C=IE"
     pub fn subject_name(&self) -> String {
         use std::ops::Deref;
-        self.value.subject_name().entries()
+        self.value
+            .subject_name()
+            .entries()
             .map(|e| {
                 let v = if let Ok(v) = e.data().as_utf8() {
                     v.deref().to_string()
@@ -349,7 +382,7 @@ impl X509 {
     }
 
     /// Gets the common name out of the cert
-    pub fn common_name(&self) -> Result<String, ()> {
+    pub fn common_name(&self) -> Result<String, X509Error> {
         self.get_subject_entry(Nid::COMMONNAME)
     }
 
@@ -389,25 +422,29 @@ impl X509 {
     fn subject_alt_names(&self) -> Option<Vec<String>> {
         if let Some(ref alt_names) = self.value.subject_alt_names() {
             // Skip the application uri
-            let subject_alt_names = alt_names.iter().skip(1).map(|n| {
-                if let Some(dnsname) = n.dnsname() {
-                    dnsname.to_string()
-                } else if let Some(ip) = n.ipaddress() {
-                    if ip.len() == 4 {
-                        let mut addr = [0u8; 4];
-                        addr[..].clone_from_slice(&ip);
-                        Ipv4Addr::from(addr).to_string()
-                    } else if ip.len() == 16 {
-                        let mut addr = [0u8; 16];
-                        addr[..].clone_from_slice(&ip);
-                        Ipv6Addr::from(addr).to_string()
+            let subject_alt_names = alt_names
+                .iter()
+                .skip(1)
+                .map(|n| {
+                    if let Some(dnsname) = n.dnsname() {
+                        dnsname.to_string()
+                    } else if let Some(ip) = n.ipaddress() {
+                        if ip.len() == 4 {
+                            let mut addr = [0u8; 4];
+                            addr[..].clone_from_slice(ip);
+                            Ipv4Addr::from(addr).to_string()
+                        } else if ip.len() == 16 {
+                            let mut addr = [0u8; 16];
+                            addr[..].clone_from_slice(ip);
+                            Ipv6Addr::from(addr).to_string()
+                        } else {
+                            "".to_string()
+                        }
                     } else {
                         "".to_string()
                     }
-                } else {
-                    "".to_string()
-                }
-            }).collect();
+                })
+                .collect();
             Some(subject_alt_names)
         } else {
             None
@@ -422,15 +459,22 @@ impl X509 {
             error!("Hostname is empty");
             StatusCode::BadCertificateHostNameInvalid
         } else if let Some(subject_alt_names) = self.subject_alt_names() {
-            let found = subject_alt_names.iter().any(|n| {
-                n.eq_ignore_ascii_case(hostname)
-            });
+            let found = subject_alt_names
+                .iter()
+                .any(|n| n.eq_ignore_ascii_case(hostname));
             if found {
                 info!("Certificate host name {} is good", hostname);
                 StatusCode::Good
             } else {
-                let alt_names = subject_alt_names.iter().map(|n| n.as_ref()).collect::<Vec<&str>>().join(", ");
-                error!("Cannot find a matching hostname for input {}, alt names = {}", hostname, alt_names);
+                let alt_names = subject_alt_names
+                    .iter()
+                    .map(|n| n.as_ref())
+                    .collect::<Vec<&str>>()
+                    .join(", ");
+                error!(
+                    "Cannot find a matching hostname for input {}, alt names = {}",
+                    hostname, alt_names
+                );
                 StatusCode::BadCertificateHostNameInvalid
             }
         } else {
@@ -442,7 +486,10 @@ impl X509 {
 
     /// Tests if the supplied application uri matches the uri alt subject name entry on the cert
     pub fn is_application_uri_valid(&self, application_uri: &str) -> StatusCode {
-        trace!("is_application_uri_valid against {} on cert", application_uri);
+        trace!(
+            "is_application_uri_valid against {} on cert",
+            application_uri
+        );
         // Expecting the first subject alternative name to be a uri that matches with the supplied
         // application uri
         if let Some(ref alt_names) = self.value.subject_alt_names() {
@@ -452,7 +499,10 @@ impl X509 {
                         info!("Certificate application uri {} is good", application_uri);
                         StatusCode::Good
                     } else {
-                        error!("Cert application uri {} does not match supplied uri {}", cert_application_uri, application_uri);
+                        error!(
+                            "Cert application uri {} does not match supplied uri {}",
+                            cert_application_uri, application_uri
+                        );
                         StatusCode::BadCertificateUriInvalid
                     }
                 } else {
@@ -477,42 +527,47 @@ impl X509 {
     ///
     /// The thumbprint might be used by the server / client for look-up purposes.
     pub fn thumbprint(&self) -> Thumbprint {
-        use openssl::hash::{MessageDigest, hash};
+        use openssl::hash::{hash, MessageDigest};
         let der = self.value.to_der().unwrap();
         let digest = hash(MessageDigest::sha1(), &der).unwrap();
         Thumbprint::new(&digest)
     }
 
     /// Turn the Asn1 values into useful portable types
-    pub fn not_before(&self) -> Result<DateTime<Utc>, ()> {
+    pub fn not_before(&self) -> Result<DateTime<Utc>, X509Error> {
         let date = self.value.not_before().to_string();
         Self::parse_asn1_date(&date)
     }
 
     /// Turn the Asn1 values into useful portable types
-    pub fn not_after(&self) -> Result<DateTime<Utc>, ()> {
+    pub fn not_after(&self) -> Result<DateTime<Utc>, X509Error> {
         let date = self.value.not_after().to_string();
         Self::parse_asn1_date(&date)
     }
 
-    pub fn to_der(&self) -> Result<Vec<u8>, ()> {
+    pub fn to_der(&self) -> Result<Vec<u8>, X509Error> {
         self.value.to_der().map_err(|e| {
             error!("Cannot turn X509 cert to DER, err = {:?}", e);
+            X509Error
         })
     }
 
-    fn parse_asn1_date(date: &str) -> Result<DateTime<Utc>, ()> {
+    fn parse_asn1_date(date: &str) -> Result<DateTime<Utc>, X509Error> {
+        const SUFFIX: &str = " GMT";
         // Parse ASN1 time format
         // MMM DD HH:MM:SS YYYY [GMT]
-        let date = if date.ends_with(" GMT") {
+        let date = if date.ends_with(SUFFIX) {
             // Not interested in GMT part, ASN1 is always GMT (i.e. UTC)
-            &date[..date.len() - 4]
+            let end = date.len() - SUFFIX.len();
+            &date[..end]
         } else {
-            &date
+            date
         };
-        Utc.datetime_from_str(date, "%b %d %H:%M:%S %Y").map_err(|e| {
-            error!("Cannot parse ASN1 date, err = {:?}", e);
-        })
+        Utc.datetime_from_str(date, "%b %d %H:%M:%S %Y")
+            .map_err(|e| {
+                error!("Cannot parse ASN1 date, err = {:?}", e);
+                X509Error
+            })
     }
 }
 
@@ -571,4 +626,3 @@ mod tests {
         })
     }
 }
-
